% Matlab M-file of Example 12.1: ex12_1.m
% Incremental fuel costs in dollars per megawatthour for a plant cinsisting of two units are given by follow.
% maximum load of each unit is 625 MW, and minimum load of each unit is 100 MW
% Total load varies from 250 ~ 1250 MW
% Find the incremental fuel cost of the plant and the allocation of load between units for the minimum cost of various tatal loads.
% Input function paferance into array.

% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %

clc
clear all
dF11dPg1 = [0.008 8];
dF21dPg2 = [0.0096 6.4];      
                                         % maximum load of each unit is 625MW
                                         % minimum load of each unit is 100MW
                      
a = dF11dPg1(1);
b = dF11dPg1(2);        
c = dF21dPg2(1);
d = dF21dPg2(2);                          % dF1/dPg1 = a*Pg1 + b = lambda1;
                                          % dF1dPg = [a b];
                                          % dF2/dPg2 = a*Pg2 + b = lambda2;
                                          % dF1dPg = [c d];

a_T=(1/a+1/c)^(-1);
b_T=a_T*(b/a+d/c);                       
PgT=100;                                 

for PgT= 250:1:1250                     
                                          % set constrain  250 <= PgT <= 1250
    lambda(PgT-249)=a_T*PgT+b_T;                   
    true_lambda(PgT-249)=lambda(PgT-249);
    Pg1(PgT-249)=(lambda(PgT-249)-b)/a;
    Pg2(PgT-249)=PgT- Pg1(PgT-249);
                                          % calculate lambda from eq(12.12)
                                          % Pg1 and Pg2 come from eq(12.11)
                                          % Supply and demand balance
    s=0;
    if Pg1(PgT-249)>625 
        Pg1(PgT-249)=Pg1(PgT-250);
        Pg2(PgT-249)=PgT-Pg1(PgT-249);
        s=1; 
                                          % Situation 1 :Unit 1 exceeds its maximum limit
                                          % Set the output of Unit 1 at its maximum limit
                                          % Reset the output of Unit 2                                      
    end
    if Pg1(PgT-249)<100 
        Pg1(PgT-249)=100;
        Pg2(PgT-249)=PgT-Pg1(PgT-249);
        s=2;
                                          % Situation 2 :Unit 1 below its minimum limit
    end
    if Pg2(PgT-249)>625 
        Pg2(PgT-249)=Pg2(PgT-250);
        Pg1(PgT-249)=PgT- Pg2(PgT-249);
        s=3;
                                          % Situation 3 :Unit 2 exceeds its maximum limit
    end
    if Pg2(PgT-249)<100 
        Pg2(PgT-249)=100;
        Pg2(PgT-249)=PgT- Pg1(PgT-249);
        s=4;
                                          % Situation 4 :Unit 2 below its minimum limit
    end
    
    PGT(PgT-249)=(PgT);
   
    if s==1
        true_lambda(PgT-249)=a*Pg2(PgT-249)+b;
    end
    if s==2
        true_lambda(PgT-249)=c*Pg2(PgT-249)+d;
    end
    if s==3
        true_lambda(PgT-249)=a*Pg1(PgT-249)+b;
    end
    if s==4
        true_lambda(PgT-249)=c*Pg1(PgT-249)+d;
    end
                                         % According to the situation 1~4 ,
                                         % recalculate the value of landa
end
figure(1)
plot(PGT,Pg1,PGT,Pg2)
title('FIGURE 12.5');
axis([0, 1400, 0, 800]);
xlabel('Plant output , (MW)');
ylabel('Unit output , (MW)');
grid on;

figure(2)
plot(PGT,true_lambda)
title('FIGURE 12.4');
axis([0, 1400, 7, 13]);
xlabel('Plant output , (MW)');
ylabel('Incremental fuel cost , ($/MWh)');
grid on;

                        
fprintf('Whan Unit 1 operating at its lower limit 100MW , the incremental cost = %1.2f \n',a*100+b)
fprintf('Whan Unit 2 operating at its lower limit 100MW , the incremental cost = %1.2f \n\n',c*100+d)

fprintf('Therefore, as plant output increases, the additional load should come from Unit 2 until equals $ %1.2f/MWh. \n',a*100+b)
fprintf('Until that point is reached, the incremental fuel cost of the plant is determined by Unit 2 alone.  \n')
fprintf('When the plant load is 250 MW, Unit 2 will supply 150 MW with equal to $%1.2f/MWh \n\n',150*c+d)

fprintf('Results are shown in Table 12.1. When is in the range from 350 to 1175 MW, the plant is determined by Eq. (12.12). At =12.4 Unit 2 is operating at its upper limit and additional load must come from Unit 1, which then determines the plant.\n')
fprintf('Figure 12.3 shows the incremental fuel cost versus plant output\n')
fprintf('Figure 12.4 shows the output of each unit versus plant output\n\n')
fprintf('TABLE 12.1\n')
fprintf('       Plant                  Unit 1           Unit 2\n')
fprintf('PgT           lambda           Pg1              Pg2\n')
fprintf('MW             $/MW            MW               MW\n')
fprintf('                 h            \n')
fprintf('------------------------------------------------------\n')
fprintf('250            %1.2f            %.0f              %.0f\n',true_lambda(250-249),Pg1(250-249),Pg2(250-249))
fprintf('350            %1.2f            %.0f              %.0f\n',true_lambda(350-249),Pg1(350-249),Pg2(350-249))
fprintf('500            %1.2f            %.0f              %.0f\n',true_lambda(500-249),Pg1(500-249),Pg2(500-249))
fprintf('700            %1.2f           %.0f              %.0f\n',true_lambda(700-249),Pg1(700-249),Pg2(700-249))
fprintf('900            %1.2f           %.0f              %.0f\n',true_lambda(900-249),Pg1(900-249),Pg2(900-249))
fprintf('1100           %1.2f           %.0f              %.0f\n',true_lambda(1100-249),Pg1(1100-249),Pg2(1100-249))
fprintf('1175           %1.2f           %.0f              %.0f\n',true_lambda(1175-249),Pg1(1175-249),Pg2(1175-249))
fprintf('1250           %1.2f           %.0f              %.0f\n',true_lambda(1250-249),Pg1(1250-249),Pg2(1250-249))


