% Matlab M-file of Example 12.3: ex12_3.m
% A thermal power plant consists of four online generating units with the
% following fuel cost functions is below.
% Input function paferance into array.

% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %
clc
clear all

F = [0.012 2.6 25;
     0.003 2.3 12;
     0.002 2.4 10;
     0.001 2.1 50];     % unit : $/hr
                        % F(Pg) = a*x^2 + b*x + c;
                        % F = [a b c];
dF1dPg = [0.024 2.6;
          0.006 2.3;
          0.004 2.4;
          0.002 2.1];   % dF/dPg = a*Pg + b = lambda;
                        % dF1dPg = [a b];
a = dF1dPg(:,1);
b = dF1dPg(:,2);

A = [dF1dPg(1,1) 0 0 0 -1;
     0 dF1dPg(2,1) 0 0 -1;
     0 0 dF1dPg(3,1) 0 -1;
     0 0 0 dF1dPg(4,1) -1;
     1 1 1 1 0];

C = [-dF1dPg(1,2);
     -dF1dPg(2,2);
     -dF1dPg(3,2);
     -dF1dPg(4,2);
     500];
 
EqSolver = A\C;
Lambda = EqSolver(5)

Pg1   = ( Lambda - b(1) ) / (a(1))
Pg2   = ( Lambda - b(2) ) / (a(2))
Pg3   = ( Lambda - b(3) ) / (a(3))
Pg4   = ( Lambda - b(4) ) / (a(4))

fprintf('The generation outputs of Unit 1 is %f MW\n',Pg1);
fprintf('The generation outputs of Unit 2 is %f MW\n',Pg2);
fprintf('The generation outputs of Unit 3 is %f MW\n',Pg3);
fprintf('The generation outputs of Unit 4 is %f MW\n',Pg4);

% By Eq(12.3) the total fuel cost of the economic dispatch schedule 
% is below

Fuel = F(1,1)*Pg1*Pg1+F(1,2)*Pg1+F(1,3)+...
       F(2,1)*Pg2*Pg2+F(2,2)*Pg2+F(2,3)+...
       F(3,1)*Pg3*Pg3+F(3,2)*Pg3+F(3,3)+...
       F(4,1)*Pg4*Pg4+F(4,2)*Pg4+F(4,3);

fprintf('Total fuel cost of the economic dispatch schedule is %f $/hr\n',Fuel);



