% Matlab M-file of Example 12.4: ex12_4.m
% Because the generation outputs of both Units 1 and 4 are not within their
% minimum and maximum generation limits, Unit 1 must fix its output at the
% minimum limit of 30MW and Unit 4 must fix its output at the maximum limit
% of 250 MW.Then, the generation outputs of Units 2 and 3 becomes
% 500-(30+250) = 220.

% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %
clc
clear all
F = [0.012 2.6 25;
     0.003 2.3 12;
     0.002 2.4 10;
     0.001 2.1 50];     % unit : $/hr
                        % F(Pg) = a*x^2 + b*x + c;
                        % F = [a b c];
dF1dPg = [0.006 2.3;
          0.004 2.4;];  % dF/dPg = a*Pg + b = lambda;
                        % dF1dPg = [a b];
a = dF1dPg(:,1);
b = dF1dPg(:,2);

A = [dF1dPg(1,1) 0 -1;
     0 dF1dPg(2,1) -1;
     1 1 0];

C = [-dF1dPg(1,2);
     -dF1dPg(2,2);
     220];
 
df1ldPg1 = [0.024 2.6];
df4ldPg4 = [0.002 2.1];
 
EqSolver = A\C;
NewLambda = EqSolver(3)

Pg1   = 30;
Pg2   = ( NewLambda - b(1) ) / (a(1));
Pg3   = ( NewLambda - b(2) ) / (a(2));
Pg4   = 250;
mun_lambda1 =  df1ldPg1(1)*Pg1 + df1ldPg1(2)
max_lambda4 =  df4ldPg4(1)*Pg4 + df4ldPg4(2)

fprintf('The incremental fuel cost of Units 2 and 3 to achieve the economic dispatch schedule is %.3f $/WMh\n',NewLambda);
fprintf('Also, Unit 1 is at its minimum generation limit and Unit 4 is at its maximum generation limit.\n');
fprintf('Then, the minimum lambda of Unit 1 is %.2f $/MWh\n',mun_lambda1);
fprintf('      the maximum lambda of Unit 4 is %.2f $/MWh\n',max_lambda4);
fprintf('The generation outputs of Unit 1 is %.0f MW\n',Pg1);
fprintf('The generation outputs of Unit 2 is %.0f MW\n',Pg2);
fprintf('The generation outputs of Unit 3 is %.0f MW\n',Pg3);
fprintf('The generation outputs of Unit 4 is %.0f MW\n',Pg4);

Fuel = F(1,1)*Pg1*Pg1+F(1,2)*Pg1+F(1,3)+...
       F(2,1)*Pg2*Pg2+F(2,2)*Pg2+F(2,3)+...
       F(3,1)*Pg3*Pg3+F(3,2)*Pg3+F(3,3)+...
       F(4,1)*Pg4*Pg4+F(4,2)*Pg4+F(4,3);

fprintf('Total fuel cost of the economic dispatch schedule is %.2f $/hr\n',Fuel);
