% Matlab M-file of Example 12.5: ex12_5.m
% This example shows a plant with a cubic function for each of its three unit fuel
% cost curves. The unit generation limits are considered and the plant load is 1000 MW. 
% The problem is solved by using bisection method for lambda iteration

% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %
clc
clear all
%% Data Input & start value
format long;

F = [6e-7 7.5e-4 6.95 1250;
     10.1e-7 1.1e-4 5.89 1300;
     1.2e-7 9e-4 6.75 720];     % unit : $/hr
                                % F(Pg) = a*x^2 + b*x + c;
                                % F = [a b c];
dF1dPg = [1.8e-6 1.5e-3 6.95;
          3.01e-6 2.2e-4 5.89;
          3.6e-7 1.8e-3 6.75];  % dF/dPg = a*Pg + b = lambda;
                                % dF1dPg = [a b];
PgUnitLimit = [100 1100;
               275 800;
               200 900];        % PgUnitLimit(:,1) minmum       
                                % PgUnitLimit(:,2) maxmum    
a = dF1dPg(:,1);
b = dF1dPg(:,2);
c = dF1dPg(:,3);

PL=1000;                        % System load
n = size(F,1);                  % Size of the dimension(1) of F.
stat = zeros(n,1);              % start statuation of Pg limit
fixStat = 1;                    % fixed/corrected statuation, 
                                % 0 for finishing / 1 for correcting and
                                % begining next iteration
Lambdamax =0;              % 0 start value
Lambdamin =inf;

for i=1:n
LDmax = a(i)*PgUnitLimit(i,2)^2+b(i)*PgUnitLimit(i,2)+c(i);
LDmin = a(i)*PgUnitLimit(i,1)^2+b(i)*PgUnitLimit(i,1)+c(i);
% if LDmax < Lambdamax            % Find minimum of PgUnit Maxmum
% Lambdamax = LDmax;end
% if LDmin > Lambdamin            % Find maximum of PgUnit Minimum
% Lambdamin = LDmin;end
if LDmax > Lambdamax            % Find minimum of PgUnit Maxmum
Lambdamax = LDmax;end
if LDmin < Lambdamin            % Find maximum of PgUnit Minimum
Lambdamin = LDmin;end
end

fprintf('Lambdamin is %.4f\n',Lambdamin);
fprintf('Lambdamax is %.4f\n',Lambdamax);

x(1) = (Lambdamax + Lambdamin)/2;   % start initial lambda value using 
                                    % bisection method

%% Iteration
fprintf('Iterations');   %printf solutions
fprintf('\t\tLambda');
for j=1:n
        fprintf('\t\tUnit%d',j);
end
fprintf('\tTotal Generation(MW)')
fprintf('\n');
while fixStat ~= 0
%% Bisection method
DiffValue = (Lambdamax - Lambdamin)/2;  % difference at the first guess
for i =1:100                            % convergence time, 30 for maximum
    j=1;                        % j for count value
    PG=0;                       % 0 for start value
    while j <= n
        if stat(j)==0           % compute the PG total from p1~3
            PG = PG + (-b(j)+sqrt(b(j)^2-4*a(j)*(c(j)-x(i))))/(2*a(j));
        end
        j = j+1;
    end
    if PG-PL > 0                    % if too more, we miner the value
        x(i+1)=x(i)-DiffValue/2;
    elseif PG-PL < 0                % if less than load, we correct the value
        x(i+1)=x(i)+DiffValue/2;
    else
        break;
    end
        %%
    
    fprintf('%d\t\t\t\t',i);
    fprintf('%.4f',x(i));
    Pgtotal=0;
    for j=1:n
        Pg(j) = (-b(j)+sqrt(b(j)^2-4*a(j)*(c(j)-x(i))))/(2*a(j));
        fprintf('\t%9.4f',Pg(j));
        Pgtotal = Pgtotal + Pg(j);
    end
    fprintf('\t\t%9.4f',Pgtotal);
    fprintf('\n');
        %%
    if abs(PG-PL) < 1e-6      % convergence value
        break;
    end
    DiffValue=DiffValue/2;          % Sisection
end

%% Determind wether if the Pg out of limit
fixStat=fixStat-1;              % fixed/corrected statuation, 
                                % 0 for finishing / 1 for correcting and
                                % begining next iteration
for k=1:n
    if stat(k) ==0 && fixStat==0
    Pg(k) = (-b(k)+sqrt(b(k)^2-4*a(k)*(c(k)-x(i))))/(2*a(k));
        if Pg(k) <= PgUnitLimit(k,1)
            stat(k)=1; % less than limit
            fixStat=fixStat+1; % fix the stat
            PL = PL - PgUnitLimit(k,1); % Change the total loss
            Pg(k) = PgUnitLimit(k,stat(k)); % fix the generation limit to the minimum
        elseif Pg(k) >= PgUnitLimit(k,2)
            stat(k)=2; % more than limit
            fixStat=fixStat+1;
            PL = PL - PgUnitLimit(k,2);
            Pg(k) = PgUnitLimit(k,stat(k)); % fix the generation limit to the maximum
        else
            stat(k)=0; % good in limit
        end
    end
end
end
Iterations = i;
fprintf('\nIterations are %d times.\n',Iterations);
Lambda = x(i);
%%
for i =1:3
fprintf('\nThe generation outputs of Unit %d is %.2f MW',i,Pg(i));
end
fprintf('\nThe Lambda is %f $/MW \n',Lambda);

Fuel = 0;
m=size(dF1dPg,2); % size of dF1dPg
for i=1:size(F,1)
    for j=0:m
        Fuel = Fuel + F(i,j+1)*Pg(i)^(m-j);
    end
end
fprintf('Total fuel cost of the economic dispatch schedule is %f $/hr\n',Fuel);
