% Matlab M-file of Example 12.6: ex12_6.m
% The 4-bus, 2-generator power system serves a load level of 500 MW or PD=5 with B-coefficint matrix provided at 100 MVA base.
% For Example 12.7, change load level to PD = 4.29 to run the same program

% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %
clc 
clear all
PD=5;
%PD=4.29;
format long

B = 0.001*[8.4122313 -0.028725 0.380697;-0.028725 5.981305 0.197142; 0.380697 0.197142 0.092345];

error(1) = 1; 
PgT(1) = 0;
lambda(1) = 0;
lambda(2) = 9.454545;

dF11dPg1 = [0.008 8];
dF21dPg2 = [0.0096 6.4];   
a = 0.5*dF11dPg1(1);
b = dF11dPg1(2);        
c = 0.5*dF21dPg2(1);
d = dF21dPg2(2);
base=100;

fprintf('The generating units at the two different power plants have incremental fuel costs in dollars per megawatthour given by follow:\n')
fprintf('df1|dPg1=0.0080Pg1+8.0 ;  df2|dPg2=0.0096*Pg2+6.4 ,  where Pg1 and Pg2 are expressed in megawatts.\n')
fprintf('At the specified load level of 500 MW the loss coefficients in per unit on a 100-MVA base are given in Example 12.3 as\n')
fprintf('[B] =[ %.6f  %.6f   %.6f;\n      %.6f   %.6f   %.6f;\n       %.6f   %.6f   %.6f ]\n',B(1,1),B(1,2),B(1,3),B(2,1),B(2,2),B(2,3),B(3,1),B(3,2),B(3,3))
fprintf('To begin the solution, we must estimate initial values for lambda for the first iteration.\n')
fprintf('The results of Example 12.1 at the 500 MW of system load can be used for this purpose. \nAssume that the specified tolerance at Step 5 for checking the power balance is erroe = 10V6.\n\n')
fprintf('Step 1 \nWe are given PD = %.2f per unit on a %.0f MVA base\n',PD,base)
fprintf('Step 2 \nFrom Example 12.1 we choose lambda(1) = 9.454545\n')
fprintf('Step 3\nBased on the estimated value of lambda(1), we calculate the outputs Pg1 and Pg2 from the equation\n')

for i=2:20

J=[0.8/lambda(i)+2*8.412231*10^-3 -2*0.028725*10^-3;-2*0.028725*10^-3 0.96/lambda(i)+2*5.981305*10^-3];
Q=[(1-0.761394*10^-3)-8/lambda(i);(1-0.394284*10^-3)-6.4/lambda(i)];

% [Q] = [J][Pg] => [Pg] = inv([j])*[Q]
Pg=J^(-1)*Q;

% Set the treansmission loss of the two units system by Eq.(12.29) 
Pg1(i)=Pg(1);
Pg2(i)=Pg(2);
PgT(i)=Pg1(i)+Pg2(i);
PL(i)=B(1,1)*Pg1(i)^2+2*B(1,2)*Pg1(i)*Pg2(i)+B(2,2)*Pg2(i)^2+2*B(1,3)*Pg1(i)+2*B(3,2)*Pg2(i)+B(3,3);

error(i)=PD+PL(i)-(Pg1(i)+Pg2(i));
delta_lambda(i)=(lambda(i)-lambda(i-1))*(error(i)/(PgT(i)-PgT(i-1)));
lambda(i+1)=lambda(i)+delta_lambda(i);

if abs(error(i)) <= 10^-6 
    break;
end
% set convergence condition
end

fprintf('Based on the estimated value of lambda(1), we calculate the outputs Pg1 and Pg2 from the equation\n')
fprintf('Pg1(1) = %.6f, Pg2(1) = %.6f per unit\n',Pg1(2),Pg2(2))
fprintf('Step 4\nFrom the results of Step 3 and the given values of the B-coefficients the system power loss is calculated as follows:\nPL = %.6f\n',PL(2))
fprintf('Step 5\nChecking the power balance for PD = 5.00 per unit, we find error = %.6f',error(2)')
fprintf('which exceeds error = 10^-6, and so a new value of lambda must be supplied. \nThe incremental change in lambda is calculated from Eq. (12.53) as follows:\n')
fprintf('Since this is the first iteration, lambda(0) and PgT(0)  are both set equal to zero, which gives\n')
fprintf('delta_lambda = %.6f, and the updated lambda then becomes %.6f\n',delta_lambda(2),lambda(3))
fprintf('Step 6\nWe now return to Step 3 to repeat the above calculations using lambda(2) during the second iteration, and so on.\n')
fprintf('The final converged solution for both system lambda and the economic loading of the two generating units is found,\nlambda= %.6f Pg1 = %.6f, Pg2 = %.6f\n',lambda(i),Pg1(i)*base,Pg2(i)*base)
fprintf('Iterative process is shown below\n')
fprintf('               Pg1                Pg2                PgT                 PL              lambda\n')
fprintf('Iterarion      (MW)              (MW)               (MW)                (MW)             ($/MWh)\n')
fprintf('start         %.4f            %.4f             %.4f              %.4f            %.4f\n',Pg1(1)*base,Pg2(1)*base,PgT(1)*base,PL(1)*base,lambda(1))
fprintf('1             %.4f          %.4f           %.4f            %.4f            %.4f\n',Pg1(2)*base,Pg2(2)*base,PgT(2)*base,PL(2)*base,lambda(2))
fprintf('2             %.4f          %.4f           %.4f            %.4f           %.4f\n',Pg1(3)*base,Pg2(3)*base,PgT(3)*base,PL(3)*base,lambda(3))
fprintf('3             %.4f          %.4f           %.4f            %.4f            %.4f\n',Pg1(4)*base,Pg2(4)*base,PgT(4)*base,PL(4)*base,lambda(4))
fprintf('4             %.4f          %.4f           %.4f            %.4f            %.4f\n',Pg1(5)*base,Pg2(5)*base,PgT(5)*base,PL(5)*base,lambda(5))
fprintf('5             %.4f          %.4f           %.4f            %.4f            %.4f\n',Pg1(6)*base,Pg2(6)*base,PgT(6)*base,PL(6)*base,lambda(6))
if (PD < 5)
    fprintf('\n')
else
    fprintf('6             %.4f          %.4f           %.4f            %.4f            %.4f\n',Pg1(7)*base,Pg2(7)*base,PgT(7)*base,PL(7)*base,lambda(7))
end

% calculate incremental losses of two plants by eq(12.30)
dPLldPg1=2*(B(1,1)*Pg1(i)+B(1,2)*Pg1(i)+B(1,3));
dPLldPg2=2*(B(2,2)*Pg2(i)+B(2,1)*Pg1(i)+B(3,1));

% calculate penalty factors of two plants by eq(12.27)
L1=1/(1-dPLldPg1);
L2=1/(1-dPLldPg2);

%  calculate incremental fuel costs of two plants
df1ldPg1=2*a*Pg1(i)*base+b;
df2ldPg2=2*c*Pg2(i)*base+d;

fprintf('The incremental losses of the Unit 1 = %.6f,\nThe incremental losses of the Unit 2 = %.6f,\n',dPLldPg1,dPLldPg2)
fprintf('and so the penalty factors are given by L1 = %.6f;   L2 = %.6f\n',L1,L2)
fprintf('The incremental fuel costs at Unit 1 are calculated = %.6f $/MWh\nThe incremental fuel costs at Unit 2 are calculated = %.6f $/MWh\n',df1ldPg1,df2ldPg2)

