% M-file for Example 12.6b- Using Newton method to solve Example 12.6: ex12_6b.m 
% The 4-bus, 2-generator power system serves a load level of 500 MW or PD=5 with B-coefficint matrix provided at 100 MVA base.
% For Example 12.7, change load level to PD = 4.29 to run the same program

% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %
clc
clear all
format long;
cost = [0   8    0.004
        0   6.4    0.0048];                      % Input cost funtion
limits=[0  Inf
          0  Inf
          0  Inf];                               % Input generation limits

% B-coefficint matrix

B=1e-3*[8.4122313 -0.028725;
       -0.028725 5.981305];
B0=1e-3*[0.380697*2 0.197142*2];
B00=1e-3*0.092345;                             % Input B, B0, B00 coefficients
PD = 5;                                         % Set total damand
%PD = 4.29;
base=100;
lambda(1)=9.454545;                             % Set initial lambda value

GenNumber = length(cost(:,1));                 
ConstantOrderTerm=cost(:,1); SecondOrderTerm=cost(:,2); HighestOrderTerm = cost(:,3);
limits= [zeros(GenNumber, 1), inf*ones(GenNumber,1)];
Pmin=limits(:,1); 
Pmax=limits(:,2);

iter = 1;                                       % Iteration counter
diff_P = 10;                                    % Setting a high initial value of diff_P
Ex=B;
fprintf('Iterations');                          % Printf solutions
fprintf('\t\tLambda');
for j=1:GenNumber
        fprintf('\t\tUnit%d',j);
end
fprintf('\t\tTotal Generation(MW)')
fprintf('\n');
while abs(diff_P)  >= 1e-3  & iter < 100         % Setting convergence condition
    for k=1:GenNumber
          Ex(k,k) =2*100* HighestOrderTerm(k)/lambda(iter) + 2*B(k,k);
          Dx(k) = (1 - B0(k)-SecondOrderTerm(k)/lambda(iter));
         for m=1:GenNumber
               if m~=k
                 Ex(k,m)=2*B(k,m);
               else,end
         end                                   
    end
    PowerUnitJacob=Ex\Dx';                      
    for k=1:GenNumber
        PowerUnitGen(k) = PowerUnitJacob(k);
    end
    PowerGenTotal(iter) = sum(PowerUnitGen);
    PL=B(1,1)*PowerUnitGen(1)^2+2*B(1,2)*PowerUnitGen(1)*PowerUnitGen(2)+B(2,2)*PowerUnitGen(2)^2+B0(1,1)*PowerUnitGen(1)+B0(1,2)*PowerUnitGen(2)+B00;
    diff_P =PD+PL -PowerGenTotal(iter);                                
    for k = 1:GenNumber
         if PowerUnitGen(k) > Pmax(k) & abs(diff_P) <=1e-3,
               PowerUnitGen(k) = Pmax(k);
         elseif PowerUnitGen(k) < Pmin(k) & abs(diff_P) <= 1e-3
               PowerUnitGen(k) = Pmin(k);
         else,  
         end           
    end
    % Check if meet the power balance constraint
    PL=B(1,1)*PowerUnitGen(1)^2+2*B(1,2)*PowerUnitGen(1)*PowerUnitGen(2)+B(2,2)*PowerUnitGen(2)^2+B0(1,1)*PowerUnitGen(1)+B0(1,2)*PowerUnitGen(2)+B00;
    diff_P =PD + PL - sum(PowerUnitGen) ;                            
    sumdp=0;
    sumbp=0;
    for i=1:GenNumber
        for j=1:GenNumber
            if j~=i
                sumbp = sumbp + B(i,j)*PowerUnitGen(j);
            end
        end
        sumdp=sumdp+(HighestOrderTerm(i)*(1-B0(1,i))+B(i,i)*SecondOrderTerm(i)-2*HighestOrderTerm(i)*sumbp)/(2*(HighestOrderTerm(i)+lambda(iter)*B(i,i))^2);
    end
   
    lambda(iter+1) = lambda(iter) + diff_P/sumdp;               
    
    fprintf('%d\t\t\t\t',iter);
    fprintf('%.4f',lambda(iter));
    Pgtotal=0;
    for j=1:GenNumber
        fprintf('\t%9.4f',PowerUnitGen(j)*base);
        Pgtotal = Pgtotal + PowerUnitGen(j);
    end
    fprintf('\t\t%10.4f',Pgtotal*base);
    fprintf('\n');
    iter = iter + 1;
end

fprintf('Iterations number = %d \n',iter-1);
fprintf('Incremental cost of delivered power (system lambda) = %10.6f $/MWh \n', lambda(iter));

for i=1:GenNumber
fprintf('Optimal Dispatch of Unit %d = %.6f MW \n',i, PowerUnitGen(i)*base);
end
fprintf('Total system loss = %g MW \n', PL*base);
