% M-file for Example 13.11: ex13_11b.m
% Transient stability of multimachine by Runge-Kutta method
% *** Copyright 2016 by G. W. Chang *** %
% *** All rights reserved *** %
clc
clear all
% ====================   Machine 1  ========================== %
%Initial values
V = 1.0; f = 60; 
%tf = 0.8;      %Final time
tf = 1.0;      %Final time
Dt = 0.05;     %Time interval
tc = 0.05;     %Clear time
global  Pm1 H1 E1 V Yf1 Yc1 f
Pm1 = 3.50; E1 = 1.100;  
Yf1 = 0;         %Admittance during the fault
Yc1 = 7.6323;    %Admittance after the fault
H1 = 11.2; 

clear t  x  delta_1
%Initial conditions
d0 = 20.82*pi/180;    %Degree -> rad
t0 = 0;
x0 = [d0 , 0];
% ======== During the fault ==========%
tspan = [t0 , tc];
[t1,xf]=ode45('df_m1', tspan, x0); 
% ======== After the fault =========%
x0c =xf(length(xf), :);
tspan = [tc , tf];        
[t2,xc] =ode45('af_m1', tspan, x0c); 

t_1 =[t1; t2]; x_1 = [xf; xc];
delta_1 = 180/pi*x_1(:,1);    %Rad -> degree

% ====================   Machine 2  ========================== %
global  Pm2 H2 E2 V Yf2 Yc2 f
Pm2 = 1.85; E2 = 1.065; 
Yf2 = 5.1665;    %Admittance during the fault
Yc2 = 6.0982;    %Admittance after the fault
H2 = 8.0; 

clear t  x  delta_2
%Initial conditions
d0 = 16.19*pi/180;    %Degree -> rad
t0 = 0;
x0 = [d0 , 0];
% ======== During the fault ==========%
tspan = [t0 , tc];
[t1,xf]=ode45('df_m2', tspan, x0); 
% ======== After the fault =========%
x0c =xf(length(xf), :);
tspan = [tc , tf];        
[t2,xc] =ode45('af_m2', tspan, x0c); 

t_2 =[t1; t2]; x_2 = [xf; xc];
delta_2 = 180/pi*x_2(:,1);    %Rad -> degree

clc
disp(' Machine 1: Fault is cleared at 0.05 Sec.')
head=['                         '
      '      time   delta_1     '
      '       s     degrees     '
      '                         '];
disp(head) 
disp([t_1, delta_1])

disp(' Machine 2 : Fault is cleared at 0.05 Sec.')
head=['                         '
      '      time   delta_2     '
      '       s     degrees     '
      '                         '];
disp(head)
disp([t_2, delta_2 ])

plot(t_1, delta_1, 'g', t_2, delta_2, 'r'), grid
title(['Multimachine system swing curve. Fault cleared at 0.05 s'])
xlabel('t, sec'), ylabel('Delta, degree')

xlim([0 1.0])
ylim([0 40])
legend('Machine 1','Machine 2');
